#pragma once

#ifndef SIMPLE_GUI_BASE_OBJECT_H
#define SIMPLE_GUI_BASE_OBJECT_H


#include "simpleColor.h"
#include "simpleGuiColor.h"
#include "simpleGuiValue.h"

#define SG_STATE_NORMAL 0
#define SG_STATE_SELECTED 1
#define SG_TYPE_FLOAT 0
#define SG_TYPE_INT 1
#define SG_TYPE_BOOL 2

class simpleGuiBaseObject{

    public:
        simpleGuiBaseObject(){
            readOnly    = false;
            bShowText   = true;
            dataType    = SG_TYPE_FLOAT;

            storedTextWidth = 0;

            //these need to be setable at some point
            fontSize     = 11;
            titleSpacing = 3;
        }


        //-------------------------------------------
        virtual bool checkHit(float x, float y){
            if(readOnly)return false;
            if( x >= hitArea.x && x <= hitArea.x + hitArea.width && y >= hitArea.y && y <= hitArea.y + hitArea.height){
                state = SG_STATE_SELECTED;
                setSelected();
                update(x, y);
                float offsetX = x - hitArea.x;
                float offsetY = y - hitArea.y;
                for(int i = 0; i < children.size(); i++){
                    children[i]->checkHit(offsetX, offsetY);
                }
                return true;
            }
            return false;
        }


        //this probably should be renamed - as this
        //is equivilant of mouse dragged once we know we
        //were hit on mouse down!
        //this is empty as it really should be specified by
        //the extending class (gui element).
        //------------------------------------------------
        virtual void update(float x, float y){

        }

         //-------------------------------------------
         void release(){
            state = SG_STATE_NORMAL;
            setNormal();
            for(int i = 0; i < children.size(); i++){
                children[i]->release();
            }
         }

        //------------------------------------------------
        virtual void setShowText(bool showText){
            bShowText = showText;
        }

        //-----------------------------------------------
        virtual void setTypeInt(){
            dataType = SG_TYPE_INT;
        }

        //-----------------------------------------------
        virtual void setTypeFloat(){
            dataType = SG_TYPE_FLOAT;
        }

        //-----------------------------------------------
        virtual void setTypeBool(){
            dataType = SG_TYPE_BOOL;
        }

        //-----------------------------------------------
        virtual void setPosition(float x, float y){
            hitArea.x     = x;
            hitArea.y     = y;
        }

        //------------------------------------------------
        virtual void setDimensions(float width, float height){
            hitArea.width       = width;
            hitArea.height      = height;
        }

        //-----------------------------------------------
        virtual float getPosX(){
            return hitArea.x;
        }

        //------------------------------------------------
        virtual float getPosY(){
            return hitArea.y;
        }

        //-----------------------------------------------
        virtual float getHeight(){
            return hitArea.height;
        }

        //------------------------------------------------
        virtual float getWidth(){
            return hitArea.width;
        }

        //--------------------------------------------
        virtual float getTextHeight(){
            float textHeight = 0;
            if(bShowText){
                textHeight += fontSize + titleSpacing;
            }
            return textHeight;
        }

        //this currently sucks because we don't know
        //the width of the text till it has been drawn!
        //--------------------------------------------
        virtual float getTextWidth(){
            float textWidth = 0;
            if(bShowText){
                 textWidth = storedTextWidth;
            }
            return  textWidth;
        }

        //a hack - guess the max possible width
        //by knowing the name and the number of values
        //--------------------------------------------
        virtual float getMaxTextWidth(){
            float textWidth = 0;
            if(bShowText){
                 textWidth = fontSize * ( name.size() + value.getNumValues() * 8 ) ;
            }
            return  textWidth;
        }


        //--------------------------------------------
        virtual void renderText(){
            if(!bShowText) return;

            glColor4fv(textColor.getColorF());
            string drawStr = name;

            for(int i = 0; i < value.getNumValues(); i++){
                if( dataType == SG_TYPE_FLOAT ) drawStr += " ["+ofToString(value.getValueF(i), 3)+"]";
                else if( dataType == SG_TYPE_INT ) drawStr += " ["+ofToString(value.getValueI(i), 0)+"]";
                else if( dataType == SG_TYPE_BOOL ) drawStr += " ["+ofToString(value.getValueB(i), 0)+"]";
            }

            //when we use real type we need to call the getStringWidth instead!
            //this is a hack and should be removed!!!!!!!!!!!!!!!!!
            storedTextWidth = drawStr.size() * fontSize;

            //need proper text options!!!!
            ofDrawBitmapString(drawStr, hitArea.x, hitArea.y - titleSpacing);
        }

        //this also needs to be specified by the extending class
        //--------------------------------------------
        virtual void render(){

        }

         //-------------------------------------------
         virtual void setSelected(){
            fgColor.setGuiColorMode(1);
            bgColor.setGuiColorMode(1);
            outlineColor.setGuiColorMode(1);
            textColor.setGuiColorMode(1);
         }

         //-------------------------------------------
         virtual void setNormal(){
            fgColor.setGuiColorMode(0);
            bgColor.setGuiColorMode(0);
            outlineColor.setGuiColorMode(0);
            textColor.setGuiColorMode(0);
         }


        //list of properties
        //------------------
        string name;

        simpleGuiValue value;

        simpleGuiColor fgColor;
        simpleGuiColor bgColor;
        simpleGuiColor outlineColor;
        simpleGuiColor textColor;

        ofRectangle boundingBox;
        ofRectangle hitArea;

        //-------------------
        vector <simpleGuiBaseObject *>children;

        float storedTextWidth;
        float titleSpacing;
        float fontSize;
        bool bShowText;
        bool readOnly;
        int  state;
        int  dataType;
};

#endif
