#ifndef SIMPLE_GUI_PANEL_H
#define SIMPLE_GUI_PANEL_H

#include "simpleGuiBaseObject.h"
#include "simpleGuiColor.h"
#include "simpleColor.h"
#include "simpleGuiValue.h"

class simpleGuiPanel : public simpleGuiBaseObject{

    public:

        //------------------------------------------------
        simpleGuiPanel(){
            currentXPos     = 20;
            currentYPos     = 20;
            spacingAmntX    = 16;
            spacingAmntY    = 16;
        }

        //------------------------------------------------
        void setup(string panelName, float defaultX = 20, float defaultY = 20){
            name = panelName;

            currentXPos = defaultX;
            currentYPos = defaultY;
        }

        //-----------------------------------------------
        void setElementSpacing(float spacingX, float spacingY){
            spacingAmntX = spacingX;
            spacingAmntY = spacingY;
        }

        //-----------------------------------------------
        //useful if you want to have two columns etc
        void resetCurrentElementPosition(float newElementPosX, float newElementPosY){
            currentXPos = newElementPosX;
            currentYPos = newElementPosY;
        }

        //-----------------------------------------------.
        void update(float x, float y){
            if( state == SG_STATE_SELECTED){
                float offsetX = x - hitArea.x;
                float offsetY = y - hitArea.y;
                for(int i = 0; i < children.size(); i++){
                    children[i]->update(offsetX, offsetY);
                }
            }
        }

        //-----------------------------------------------
        void addElement( simpleGuiBaseObject * element ){
            children.push_back( element );
        }


        //-----------------------------------------------
        void checkResize(simpleGuiBaseObject * element){
             //if the object is off the panel in x - resize the panel
            float checkWidth = element->getPosX() + MAX(element->getWidth(), element->getMaxTextWidth());

            if(checkWidth >= hitArea.width ){
                hitArea.width += ( checkWidth - hitArea.width ) + spacingAmntX;
            }

            //if the object is off the panel in y - resize the panel
            float checkHeight = element->getHeight() + element->getPosY();

            if(checkHeight >= hitArea.height ){
                hitArea.height += ( checkHeight - hitArea.height ) + spacingAmntY;
            }
        }


        //-----------------------------------------------
        void addElementHorizontal( simpleGuiBaseObject * element ){
            element->setPosition(currentXPos, currentYPos);

            //add the element to the panel list
            children.push_back( element );

            //update the current position for the next element
            currentXPos += element->getWidth() + spacingAmntX;

            //see if we need to resize!
            checkResize(element);
        }


        //-----------------------------------------------
        void addElementVertical( simpleGuiBaseObject * element ){

            //if there is text - we need to shift down first
            currentYPos += element->getTextHeight();
            element->setPosition(currentXPos, currentYPos);

            //add the element to the panel list
            children.push_back( element );

            //update the current position for the next element
            currentYPos += element->getHeight() + spacingAmntY;

            //see if we need to resize!
            checkResize(element);
        }

        //-----------------------------------------------.
        void render(){
            glPushMatrix();
            glTranslatef(hitArea.x, hitArea.y, 0);

                //draw the background
                ofFill();
                glColor4fv(bgColor.getColorF());
                ofRect(0, 0, hitArea.width, hitArea.height);

                //draw the outline
                ofNoFill();
                glColor4fv(outlineColor.getColorF());
                ofRect(0, 0, hitArea.width, hitArea.height);

                for(int i = 0; i < children.size(); i++){
                    children[i]->render();
                }

            glPopMatrix();

            simpleGuiBaseObject::renderText();
        }


        float currentXPos;
        float currentYPos;
        float spacingAmntX;
        float spacingAmntY;


};

#endif
